
------------------------------------------------------------------------
--
-- AbsorbsMonitor
--
-- Copyright (C) 2010  Philipp Schmidt
--
-- This program is free software; you can redistribute it and/or
-- modify it under the terms of the GNU General Public License
-- as published by the Free Software Foundation; either version 2
-- of the License, or (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to:
--
-- Free Software Foundation, Inc.,
-- 51 Franklin Street, Fifth Floor,
-- Boston, MA  02110-1301, USA.
--
--
------------------------------------------------------------------------


local AM_Public = _G.AbsorbsMonitor;
local AM_Core = AM_Public.GetCoreObject();

local AceConfig = LibStub("AceConfig-3.0");
local AceConfigDialog = LibStub("AceConfigDialog-3.0");
local AceDB = LibStub("AceDB-3.0");

local L = LibStub("AceLocale-3.0"):GetLocale("AbsorbsMonitor");


-- #################################################################################################


local function getProfileOption(info)
	return AM_Core.db.profile[info.arg];
end

local function setProfileOption(info, value)
	AM_Core.db.profile[info.arg] = value;
end


-- #################################################################################################


local defaults = {
	profile = {
		passive = false,
		silent = false,	
		zoneModifier = 1.3,
		zoneModifier_auto = true,
		lowValueTolerance = 50,
	},
};


-- #################################################################################################


local options = {
	type = "group",
	get = getProfileOption,
	set = setProfileOption,
	
	args = {
		general = {
			type = "group",
			name = L["General Options"],
			cmdInline = true,
			order = -1,
			
			args = {
				options = {
					type = "execute",
					order = 1,
					name = L["Options Window"],
					desc = L["Shows the Options Window"],
					func = function()
						AM_Core:ShowOptions();
					end,
					guiHidden = true,
				},			
				enabled = {
					type = "toggle",
					order = 2,
					name = L["Enabled"],
					desc = L["Enables/Disables this addon"],
					get = function() return AM_Public:IsEnabled() end,
					set = function()
						if(AM_Public:IsEnabled()) then
							AM_Public:Disable();
						else
							AM_Public:Enable()
						end
					end,
				},
				passive = {
					type = "toggle",
					order = 3,
					name = L["Passive"],
					desc = L["Switches to passive mode, disabling all combat log parsing"],
					arg = "passive",
					set = function(info, value)
						if(value) then
							AM_Core:SetPassive();							
						else
							AM_Core:SetActive();
						end
					end,
				},
				silent = {
					type = "toggle",
					order = 4,
					name = L["Silent"],
					desc = L["Switches to silent mode, disabling all addon communication"],
					arg = "silent",
					set = function(info, value)
						if(value) then
							AM_Core:SetSilent();
						else
							AM_Core:SetVerbose();
						end
					end,
				},
				paramHeader = {
					type = "header",
					order = 5,
					name = L["Parameters"],
				},
				zonemod = {
					type = "range",
					order = 5,
					min = 0,
					max = 2.0,
					softMin = 0.9,
					softMax = 1.3,
					name = L["Zone Modifier"],
					desc = L["Global factor on absorb effects like the ICC Buff or the PvP Penalty"],
					arg = "zoneModifier",
					set = function(info, value)
						setProfileOption(info, value);
						AM_Core.SetZoneModifier(value);
					end,
				},
				zonemod_auto = {
					type = "toggle",
					order = 6,
					name = L["Automatic Zone Modifier"],
					desc = L["Adjust the zone modifier automatically"],
					arg = "zoneModifier_auto",				
					set = function(...)
						setProfileOption(...);
						AM_Core.OnZoneChangedNewArea();
					end,
				},		
				lowcap = {
					type = "range",
					min = 0,
					max = 10000,
					softMin = 0,
					softMax = 250,					
					name = L["Low Value Tolerance"],
					desc = L["Accepted tolerance for underestimating the absorb value"],
					arg = "lowValueTolerance",
					set = function(info, value)
						setProfileOption(info, value);
						AM_Core.SetLowValueTolerance(value);
					end,
				},						
			},
		},
	},
};


-- #################################################################################################


-- Update the local cached values
local function OnProfileChanged()
	local profile = AM_Core.db.profile;

	AM_Core.SetLowValueTolerance(profile.lowValueTolerance);
	AM_Core.SetZoneModifier(profile.zoneModifier);
end

function AM_Core:InitOptions()
	self.db = AceDB:New("AbsorbsMonitorDB", defaults);
	
	self.db.RegisterCallback(self, "OnProfileChanged", OnProfileChanged);
	self.db.RegisterCallback(self, "OnProfileCopied", OnProfileChanged);
	self.db.RegisterCallback(self, "OnProfileReset", OnProfileChanged);
	
	options.args.profile = LibStub("AceDBOptions-3.0"):GetOptionsTable(self.db);
	options.args.profile.order = -2;
	
	AceConfig:RegisterOptionsTable("AbsorbsMonitor", options, {"am", "absorbsmonitor"});
	
	self.optionsFrames = {}
	self.optionsFrames.general = AceConfigDialog:AddToBlizOptions("AbsorbsMonitor", nil, nil, "general");
	self.optionsFrames.profile = AceConfigDialog:AddToBlizOptions("AbsorbsMonitor", L["Profiles"], "AbsorbsMonitor", "profile");
end

function AM_Core:ShowOptions()
	InterfaceOptionsFrame_OpenToCategory(self.optionsFrames.profile);
	InterfaceOptionsFrame_OpenToCategory(self.optionsFrames.general);
end
