
------------------------------------------------------------------------
--
-- ShieldFrames
--
-- Copyright (C) 2010  Philipp Schmidt
--
-- This program is free software; you can redistribute it and/or
-- modify it under the terms of the GNU General Public License
-- as published by the Free Software Foundation; either version 2
-- of the License, or (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to:
--
-- Free Software Foundation, Inc.,
-- 51 Franklin Street, Fifth Floor,
-- Boston, MA  02110-1301, USA.
--
--
------------------------------------------------------------------------


local ShieldFrames = _G.ShieldFrames;

local AnchorProto = {};
ShieldFrames.AnchorProto = AnchorProto;

local AceRegistry = LibStub("AceConfigRegistry-3.0");

local playerGUID;


local backdrop = {bgFile = "Interface\\Tooltips\\UI-Tooltip-Background",
		edgeFile = "Interface\\ChatFrame\\ChatFrameBackground", edgeSize = 0.90,
		insets = {left = 1, right = 1, top = 1, bottom = 1}};


local function NoOp()
end

local function OnDragStop(self)
	self:StopMovingOrSizing();

	local x, y = self:GetLeft(), self:GetTop();
		
	self.profile.posX, self.profile.posY = x, y;
	
	AceRegistry:NotifyChange("ShieldFrames");
end

function AnchorProto.Enable()
	playerGUID = UnitGUID("player");
end

function AnchorProto.defaults()
	return {
		type = "ungrouped",
		posX = 600,
		posY = 600,
		scale = 1.0,
		width = 200,
		growthDirection = -1,
		anchorHeight = 20,
		effectHeight = 16,
		durationHeight = 3,
		anchorAlpha = 1.0,
		effectAlpha = 1.0,
		effectSpacing = 3,
		textHeight = 8,
		textFormat = "%d - %n|%a (%p%)",
		anchorVisible = true,
		filterAppliedByMe = false,
		filterAppliedOnMe = false,
	};
end

function AnchorProto:New(name, profile)
	local o = setmetatable({}, self);
	self.__index = self;
	
	o.name = name;	
	o.profile = profile or self.defaults();	
	o.effects = {};

	o:SetupType();
	o:CreateFrame();
	
	return o;
end

function AnchorProto:Release()
	for guid, guidEffects in pairs(self.effects) do
		for spellId, effect in pairs(guidEffects) do
			effect:Release();
		end
	end
	
	self:Hide();
end

function AnchorProto:SetupType()
	if(self.profile.type == "ungrouped") then
		self.Absorbs_UnitUpdated = NoOp;
		
	elseif(self.profile.type == "byUnit") then
	
	elseif(self.profile.type == "bySpell") then
	end
end

function AnchorProto:CreateFrame()
	local frame = CreateFrame("Frame", nil, UIParent);
	self.frame = frame;
	
	-- This has to go, probably I will merge the frame object into
	-- the AnchorProto object itself
	frame.profile = self.profile;	
	
	frame:EnableMouse(true);
	frame:SetScript("OnDragStart", self.frame.StartMoving);
	frame:SetScript("OnDragStop", OnDragStop);	
	
	frame:SetBackdrop(backdrop);
	frame:SetBackdropColor(0, 0, 0, 0.5);
	frame:SetBackdropBorderColor(1, 1, 1, 0.6);
	
	frame.label = frame:CreateFontString(nil, "OVERLAY");
	frame.label:SetPoint("LEFT", frame, 6, 0);
	frame.label:SetFontObject(GameFontHighlightSmall);
		
	frame:Hide();
	
	self:ApplyProfile();
		
	self.lastChild = self;
end

function AnchorProto:ApplyProfile()
	local profile = self.profile;
	local frame = self.frame;	

	frame:SetScale(profile.scale);		
	frame:SetWidth(profile.width);
	frame:SetHeight(profile.anchorHeight);
	frame:SetAlpha(profile.anchorAlpha);
	
	frame.label:SetText(self.name);
	
	frame:ClearAllPoints();
	frame:SetPoint("TOPLEFT", UIParent, "BOTTOMLEFT", profile.posX, profile.posY);	
	
	for guid, guidEffects in pairs(self.effects) do
		for spellId, effect in pairs(guidEffects) do
			effect:ApplyProfile();
		end
	end
	
	self:Hide();
	self:ShowIfVisible();
end

function AnchorProto:Lock()
	self.frame:SetMovable(false);
	self.frame:RegisterForDrag();
	
	self:Hide();
	self:ShowIfVisible();
end

function AnchorProto:Unlock()
	self.frame:SetMovable(true);	
	self.frame:RegisterForDrag("LeftButton");	
	
	self:Show(true);
end

function AnchorProto:ReanchorEffects()
	local cur = self.next;

	-- Just make sure we do not get an infinite loop through some weird kind
	-- of circular reference
	local i = 0;
	
	while(cur) do
		if(i == 500) then
			error("Loop counter on reanchoring reached 500, circular effect reference?");
		end
		
		i = i + 1;
		
		cur:AlignTo(cur.prev);
		cur = cur.next;
	end
end

function AnchorProto:Absorbs_EffectApplied(sourceGUID, sourceName, destGUID, destName, spellId, amount, quality, duration)
	local profile = self.profile;
	
	if((sourceGUID ~= playerGUID) and (profile.filterAppliedByMe)) then return; end
	if((destGUID ~= playerGUID) and (profile.filterAppliedOnMe)) then return; end

	local effect = ShieldFrames.EffectProto:New(profile);
	
	effect:SetupValues(sourceName, destName, spellId, amount, quality, duration);
	
	effect.prev = self.lastChild;
	self.lastChild.next = effect;
		
	effect:AlignTo(self.lastChild);
	
	self.lastChild = effect;
	
	if(not self.effects[destGUID]) then
		self.effects[destGUID] = {[spellId] = effect};
	else
		self.effects[destGUID][spellId] = effect;
	end		
	
	effect.frame:Show();
end

function AnchorProto:Absorbs_EffectUpdated(guid, spellId, amount, quality, duration)
	local guidEffects = self.effects[guid];

	if(guidEffects and guidEffects[spellId]) then
		guidEffects[spellId]:UpdateValues(amount, quality, duration);
	end
end

function AnchorProto:Absorbs_EffectRemoved(guid, spellId)
	local guidEffects = self.effects[guid];
	
	if(guidEffects and guidEffects[spellId]) then
		local effect = guidEffects[spellId];
		
		local prev = effect.prev;
		local next = effect.next;
		
		effect:Release();
		
		if(self.lastChild == effect) then
			self.lastChild = prev;
		end
		
		prev.next = next;
		
		if(next) then
			next.prev = prev;		
			next:AlignTo(prev);
		end
		
		guidEffects[spellId] = nil;
	end
end

function AnchorProto:Absorbs_UnitUpdated(guid, amount, quality)
	-- NoOp for ungrouped
end

function AnchorProto:Absorbs_UnitCleared(guid)
	local guidEffects = self.effects[guid];
	
	if(guidEffects) then
		for spellId, effect in pairs(guidEffects) do
			-- A little inefficient here, should optimize that later
			-- But then again, it's rare to have such a message on a
			-- non-empty unit
			self:Absorbs_EffectRemoved(guid, spellId);
		end
	end
end

function AnchorProto:Absorbs_AreaCreated(sourceGUID, sourceName, triggerGUID, spellId, value, quality)
	-- AREA support later
end

function AnchorProto:Absorbs_AreaUpdated(triggerGUID, value)
	-- AREA support later
end

function AnchorProto:Absorbs_AreaCleared(triggerGUID)
	-- AREA support later
end

function AnchorProto:ShowIfVisible()
	self:Show();
end

function AnchorProto:Show(force)
	if(self.profile.anchorVisible or force) then
		self.frame:Show();
	end
end

function AnchorProto:Hide()
	self.frame:Hide();
end
